@@ -10,7 +10,7 @@ use std::rc::Rc;
 type HeaderSignature = [u8; 8];
 
 const _DAT_HEADER_SZ: u64 = 128;
-const DAT_HEADER_SIGNATURE: HeaderSignature = [b's', b'h', b'a', b'm', b'd', b'b', b'0', 0u8];
+const DAT_HEADER_SIGNATURE: HeaderSignature = [b's', b'i', b'a', b'm', b'd', b'b', b'0', 0u8];
 
 #[derive(Debug, Clone)]
 pub struct DatFile(Rc<RefCell<VarFile>>);
@@ -421,7 +421,14 @@ fn dat_write_record(
             RecordOffset::new(file.stream_position()?)
         };
         file.write_record_size(new_record_size)?;
-        file.write_all(buf_ref)?;
+        match file.write_all(buf_ref) {
+            Ok(()) => (),
+            Err(err) => {
+                // recover on error
+                let _ = file.set_len(new_record_offset);
+                return Err(err);
+            }
+        }
         Ok(new_record_offset)
     }
 }
