@@ -1,5 +1,6 @@
 use super::buf::BufFile;
 use super::semtype::*;
+use std::convert::TryInto;
 use std::fs::File;
 use std::io::{Cursor, Read, Result, Seek, SeekFrom, Write};
 
@@ -54,6 +55,26 @@ impl VarFile {
         self.seek(SeekFrom::Start(offset.as_value()))
             .map(Offset::<T>::new)
     }
+    ///
+    pub fn set_len<T>(&mut self, size: Offset<T>) -> Result<()> {
+        self.buf_file.set_len(size.as_value())
+    }
+    ///
+    pub fn write_all_small(&mut self, buf: &mut [u8]) -> Result<()> {
+        self.buf_file.write_all_small(buf)
+    }
+    ///
+    pub fn write_zero(&mut self, size: usize) -> Result<()> {
+        self.buf_file.write_zero(size)
+    }
+    ///
+    pub fn write_node_clear(&mut self, node_offset: NodeOffset, node_size: NodeSize) -> Result<()> {
+        let _ = self.seek_from_start(node_offset)?;
+        self.write_zero(node_size.try_into().unwrap())?;
+        let _ = self.seek_from_start(node_offset)?;
+        self.write_node_size(node_size)?;
+        Ok(())
+    }
 }
 
 impl Read for VarFile {
@@ -505,50 +526,23 @@ impl VarCursor {
 //--
 #[cfg(test)]
 mod debug {
+    use super::VarFile;
+    //
     #[test]
     fn test_size_of() {
-        use super::VarFile;
-        //
         #[cfg(target_pointer_width = "64")]
-        #[cfg(not(feature = "buf_stats"))]
         {
-            #[cfg(feature = "vf_u32u32")]
+            #[cfg(not(feature = "buf_stats"))]
             assert_eq!(std::mem::size_of::<VarFile>(), 120);
-            #[cfg(feature = "vf_u64u64")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 120);
-            #[cfg(feature = "vf_vu64")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 120);
-        }
-        #[cfg(target_pointer_width = "32")]
-        #[cfg(not(feature = "buf_stats"))]
-        {
-            #[cfg(feature = "vf_u32u32")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 76);
-            #[cfg(feature = "vf_u64u64")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 76);
-            #[cfg(feature = "vf_vu64")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 76);
-        }
-        //
-        #[cfg(target_pointer_width = "64")]
-        #[cfg(feature = "buf_stats")]
-        {
-            #[cfg(feature = "vf_u32u32")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 128);
-            #[cfg(feature = "vf_u64u64")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 128);
-            #[cfg(feature = "vf_vu64")]
+            #[cfg(feature = "buf_stats")]
             assert_eq!(std::mem::size_of::<VarFile>(), 128);
         }
         #[cfg(target_pointer_width = "32")]
-        #[cfg(feature = "buf_stats")]
         {
-            #[cfg(feature = "vf_u32u32")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 76);
-            #[cfg(feature = "vf_u64u64")]
-            assert_eq!(std::mem::size_of::<VarFile>(), 76);
-            #[cfg(feature = "vf_vu64")]
+            #[cfg(not(feature = "buf_stats"))]
             assert_eq!(std::mem::size_of::<VarFile>(), 76);
+            #[cfg(feature = "buf_stats")]
+            assert_eq!(std::mem::size_of::<VarFile>(), 84);
         }
     }
 }
